@@ -801,10 +801,27 @@ ev_sidebar_thumbnails_device_scale_factor_changed_cb (EvSidebarThumbnails *sideb
         ev_sidebar_thumbnails_reload (sidebar_thumbnails);
 }
 
+static void
+ev_sidebar_thumbnails_row_changed (GtkTreeModel *model,
+                                   GtkTreePath  *path,
+                                   GtkTreeIter  *iter,
+                                   gpointer      data)
+{
+	guint signal_id;
+
+	signal_id = GPOINTER_TO_UINT (data);
+
+	/* PREVENT GtkIconView "row-changed" handler to be reached, as it will
+	 * perform a full invalidate and relayout of all items, See bug:
+	 * https://bugzilla.gnome.org/show_bug.cgi?id=691448#c9 */
+	g_signal_stop_emission (model, signal_id, 0);
+}
+
 static void
 ev_sidebar_thumbnails_init (EvSidebarThumbnails *ev_sidebar_thumbnails)
 {
 	EvSidebarThumbnailsPrivate *priv;
+	guint signal_id;
 
 	priv = ev_sidebar_thumbnails->priv = EV_SIDEBAR_THUMBNAILS_GET_PRIVATE (ev_sidebar_thumbnails);
 
@@ -814,6 +831,11 @@ ev_sidebar_thumbnails_init (EvSidebarThumbnails *ev_sidebar_thumbnails)
 					       G_TYPE_BOOLEAN,
 					       EV_TYPE_JOB_THUMBNAIL);
 
+	signal_id = g_signal_lookup ("row-changed", GTK_TYPE_TREE_MODEL);
+	g_signal_connect (GTK_TREE_MODEL (priv->list_store), "row-changed",
+			  G_CALLBACK (ev_sidebar_thumbnails_row_changed),
+			  GUINT_TO_POINTER (signal_id));
+
 	priv->swindow = gtk_scrolled_window_new (NULL, NULL);
 
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->swindow),
@@ -962,6 +984,8 @@ thumbnail_job_completed_callback (EvJobThumbnail      *job,
 			    COLUMN_JOB, NULL,
 			    -1);
         cairo_surface_destroy (surface);
+
+        gtk_widget_queue_draw (priv->icon_view);
 }
 
 static void
@@ -982,10 +1006,14 @@ ev_sidebar_thumbnails_document_changed_cb (EvDocumentModel     *model,
 	priv->n_pages = ev_document_get_n_pages (document);
 	priv->rotation = ev_document_model_get_rotation (model);
 	priv->inverted_colors = ev_document_model_get_inverted_colors (model);
-	priv->loading_icons = g_hash_table_new_full (g_str_hash,
-						     g_str_equal,
-						     (GDestroyNotify)g_free,
-						     (GDestroyNotify)cairo_surface_destroy);
+	if (priv->loading_icons) {
+                g_hash_table_remove_all (priv->loading_icons);
+	} else {
+                priv->loading_icons = g_hash_table_new_full (g_str_hash,
+                                                             g_str_equal,
+                                                             (GDestroyNotify)g_free,
+                                                             (GDestroyNotify)cairo_surface_destroy);
+	}
 
 	ev_sidebar_thumbnails_clear_model (sidebar_thumbnails);
 	ev_sidebar_thumbnails_fill_model (sidebar_thumbnails);
